from django_pay2.decorators import handle_debug
from django_pay2.models import Payment
from datetime import timedelta

from .api import get_api


@handle_debug
def create_qiwi_payment(request, amount, desc, receiver, currency, expirate_at=None):
    payment = Payment.objects.create(amount=amount, receiver=receiver)
    expirate_at = expirate_at or payment.created + timedelta(days=1)
    api = get_api()
    return api.generate_payment_method(amount, currency, expirate_at, payment.id)



from typing import List, Optional

from django.urls import reverse
from ipware.ip import get_client_ip

from django.utils.translation import get_language
from django_pay2.decorators import handle_debug
from django_pay2.exceptions import CreatePaymentError
from django_pay2.models import Payment
from django_pay2.settings import payment_settings
from django_pay2.payment_methods import PaymentRedirect

from .api import TinkoffOrderItem, TinkoffApiError
from .services import get_tinkoff_api


@handle_debug
def create_tinkoff_payment(
    request,
    amount,
    desc,
    receiver,
    items: List[TinkoffOrderItem],
    client_email: Optional[str] = None,
    client_phone: Optional[str] = None,
):
    payment = Payment.objects.create(amount=amount, receiver=receiver)
    try:
        api = get_tinkoff_api()
        notification_url = reverse("django_pay2:tinkoff:notify")
        success_url = reverse("django_pay2:success")
        fail_url = reverse("django_pay2:fail")
        result = api.init_payment(
            amount,
            str(payment.id),
            get_client_ip(request)[0],
            desc,
            get_language(),
            notification_url,
            success_url,
            fail_url,
            items=items,
            client_email=client_email,
            client_phone=client_phone,
            email_company=payment_settings.TINKOFF.email_company,
            taxation=payment_settings.TINKOFF.taxation,
        )
        return PaymentRedirect(result.payment_url)
    except TinkoffApiError as exc:
        payment.reject()
        raise CreatePaymentError(str(exc))



from rest_framework import serializers

from django_pay2.models import Payment
from django_pay2.utils import MappingFieldsMixin

from .services import get_tinkoff_api


class RawDataDefault:
    requires_context = True

    def __call__(self, serializer_field):
        return serializer_field.context["request"].data

    def __repr__(self):
        return "%s()" % self.__class__.__name__


class NotifySerializer(MappingFieldsMixin, serializers.Serializer):
    terminal_key = serializers.CharField()
    token = serializers.CharField()
    payment = serializers.PrimaryKeyRelatedField(queryset=Payment.objects.all())
    is_success = serializers.BooleanField()
    amount_rub = serializers.IntegerField()
    raw_data = serializers.HiddenField(default=RawDataDefault())

    mapping_keys = {
        "terminal_key": "TerminalKey",
        "token": "Token",
        "payment": "OrderId",
        "is_success": "Success",
        "amount_rub": "Amount",
    }

    default_error_messages = {
        "unexpected_terminal_token": "Unexpected terminal token",
        "must_be_success": "Payment must be success",
        "mismatch_token": "Mismatch token",
        "mismatch_amount": "Mismatch amount",
    }

    def validate_terminal_key(self, key):
        if get_tinkoff_api().terminal_key != key:
            self.fail("unexpected_terminal_key")
        return key

    def validate_is_success(self, is_success):
        if not is_success:
            self.fail("must_be_success")
        return is_success

    def validate_amount_rub(self, amount):
        return amount * 100

    def validate(self, attrs):
        attrs = super().validate(attrs)

        expected_token = get_tinkoff_api().calculate_token(attrs["raw_data"])
        if attrs["token"] != expected_token:
            self.fail("mismatch_token")

        if attrs["payment"].amount != attrs["amount_rub"]:
            self.fail("mismatch_amount")

        return attrs



from enum import Enum


class Languages(str, Enum):
    RU = "ru"
    EN = "en"


class PayTypes(str, Enum):
    O = "O"  # noqa
    T = "T"


class Taxations(str, Enum):
    OSN = "osn"
    USN_INCOME = "usn_income"
    USN_INCOME_OUTCOME = "usn_income_outcome"
    PATENT = "patent"
    ENVD = "envd"
    ESN = "esn"


class PaymentMethods(str, Enum):
    FULL_PAYMENT = "full_payment"
    FULL_PREPAYMENT = "full_prepayment"
    PREPAYMENT = "prepayment"
    ADVANCE = "advance"
    PARTIAL_PAYMENT = "partial_payment"
    CREDIT = "credit"
    CREDIT_PAYMENT = "credit_payment"


class Taxes(str, Enum):
    NONE = "none"
    VAT0 = "vat0"
    VAT10 = "vat10"
    VAT20 = "vat20"
    VAT110 = "vat110"
    VAT120 = "vat120"


class PaymentObjects(str, Enum):
    COMMODITY = "commodity"
    EXCISE = "excise"
    JOB = "job"
    SERVICE = "service"
    GAMBLING_BET = "gambling_bet"
    GAMBLING_PRIZE = "gambling_price"
    LOTTERY = "lottery"
    LOTTERY_PRIZE = "lottery_prize"
    INTELLECTUAL_ACTIVITY = "intellectual_activity"
    PAYMENT = "payment"
    AGENT_COMMISSION = "agent_commission"
    COMPOSITE = "composite"
    ANOTHER = "another"



from django_pay2.decorators import handle_debug
from django_pay2.models import Payment
from django_pay2.settings import payment_settings
from django.http import HttpRequest
from django.db.models import Model
from typing import Any, Optional
from django_pay2.payment_methods import PaymentRedirect

from .api import SberbankApi


def get_api() -> SberbankApi:
    return SberbankApi(
        username=payment_settings.SBERBANK.username,
        password=payment_settings.SBERBANK.password,
    )


@handle_debug
def create_sberbank_payment(
    request: HttpRequest,
    amount: Any,
    desc: str,
    receiver: Model,
    return_url: Optional[str] = None,
    fail_url: Optional[str] = None,
    page_view: Optional[str] = None,
    phone: Optional[str] = None,
) -> PaymentRedirect:
    return_url = return_url or payment_settings.SBERBANK.return_url
    fail_url = fail_url or payment_settings.SBERBANK.fail_url
    payment = Payment.objects.create(amount=amount, receiver=receiver)
    api = get_api()
    return api.register_payment(
        order_num=str(payment.id),
        amount=str(amount),
        return_url=return_url,
        fail_url=fail_url,
        description=desc,
        page_view=page_view,
        phone=phone,
    )



from django_pay2.settings import payment_settings

from .api import PayeerApi


def get_payeer_api() -> PayeerApi:
    return PayeerApi(
        account=payment_settings.PAYEER.account,
        api_id=payment_settings.PAYEER.api_id,
        api_password=payment_settings.PAYEER.api_password,
        secret_key=payment_settings.PAYEER.secret_key,
        shop_id=payment_settings.PAYEER.shop_id,
    )



def clear_none(a_dict):
    return {key: value for key, value in a_dict.items() if value is not None}


class MappingFieldsMixin:
    mapping_keys = {}

    def to_internal_value(self, data):
        mapped_data = {
            new_key: data.get(old_key) for new_key, old_key in self.mapping_keys.items()
        }
        return super().to_internal_value(
            {
                **data,
                **mapped_data,
            }
        )


def remove_trivial_zeros(num):
    num = str(num)
    return num.rstrip("0").rstrip(".")



from django.db.models import TextChoices, IntegerChoices


class SberbankCallbackOperation(TextChoices):
    approved = "approved"
    declined_by_timeout = "declinedByTimeout"
    deposited = "deposited"
    reversed = "reversed"
    refunded = "refunded"


class SberbankCallbackStatus(IntegerChoices):
    success = 1
    error = 0



import requests
from urllib.parse import urljoin
from typing import Optional

from django_pay2.payment_methods import PaymentRedirect

from .exceptions import SberbankApiError


def clear_none(d: dict) -> dict:
    return {key: value for key, value in d.items() if value is not None}


class SberbankApi:
    base_url = "https://securepayments.sberbank.ru/payment/"

    def __init__(self, username: str, password: str):
        self.username = username
        self.password = password

    def register_payment(
        self,
        *,
        order_num: str,
        amount: str,
        return_url: str,
        fail_url: str,
        description: Optional[str] = None,
        page_view: Optional[str] = None,
        phone: Optional[str] = None,
    ) -> PaymentRedirect:
        r = requests.post(
            self.build_url("rest/register.do"),
            params=clear_none(
                {
                    "userName": self.username,
                    "password": self.password,
                    "orderNumber": order_num,
                    "amount": amount,
                    "returnUrl": return_url,
                    "failUrl": fail_url,
                    "description": description,
                    "pageView": page_view,
                    "phone": phone,
                }
            ),
        )
        if not r.ok:
            raise SberbankApiError(r)
        data = r.json()
        if data.get("errorCode"):
            raise SberbankApiError(r)
        return PaymentRedirect(data["formUrl"])

    def build_url(self, path: str) -> str:
        return urljoin(self.base_url, path)



from django.urls import include, path

from . import views

app_name = "django_pay2"

urlpatterns = [
    path("success/", views.SuccessPaymentView.as_view(), name="success"),
    path("fail/", views.RejectedPaymentView.as_view(), name="fail"),
    path(
        "debug_payments/<uuid:pk>/",
        views.DebugPaymentView.as_view(),
        name="debug_payment",
    ),
    path(
        "debug_payments/<uuid:pk>/accept/",
        views.AcceptDebugPaymentView.as_view(),
        name="debug_accept",
    ),
    path(
        "debug_payments/<uuid:pk>/reject/",
        views.RejectDebugPaymentView.as_view(),
        name="debug_reject",
    ),
    path("tinkoff/", include("django_pay2.providers.tinkoff.urls")),
    path("payeer/", include("django_pay2.providers.payeer.urls")),
    path("perfect-money/", include("django_pay2.providers.perfect_money.urls")),
    path("coinpayments/", include("django_pay2.providers.coinpayments.urls")),
    path("qiwi/", include("django_pay2.providers.qiwi.urls")),
    path("free-kassa/", include("django_pay2.providers.free_kassa.urls")),
    path("sberbank/", include("django_pay2.providers.sberbank.urls")),
]



class TinkoffApiError(Exception):
    pass


class TinkoffNetworkError(TinkoffApiError):
    def __init__(self, status_code, text):
        self.status_code = status_code
        self.text = text

    def __str__(self):
        return f"status_code={self.status_code},text={self.text}"


class TinkoffResponseError(TinkoffApiError):
    def __init__(self, error_code, message=None, details=None):
        self.error_code = error_code
        self.message = message
        self.details = details

    def __str__(self):
        ret = f"error_code={self.error_code}"
        if self.message:
            ret += f",message={self.message}"
        if self.details:
            ret += f",details={self.details}"
        return ret



from rest_framework import generics, status
from rest_framework.response import Response

from .serializers import QiwiNotifySerializer


class QiwiNotifyView(generics.GenericAPIView):
    def post(self, request, *args, **kwargs):
        serializer = QiwiNotifySerializer(
            data=request.data,
            context={
                "hmac": request.META.get("HTTP_X_API_SIGNATURE_SHA256"),
            },
        )
        serializer.is_valid(raise_exception=True)
        serializer.validated_data["bill"]["bill_id"].accept()
        return Response(status=status.HTTP_204_NO_CONTENT)



from typing import List, Optional


class PayeerError(Exception):
    def __init__(self, errors: List[str]):
        self.errors = errors

    def __str__(self):
        return str(self.errors)


class PayeerValidationError(Exception):
    def __init__(self, msg: str, order_id: Optional[str] = None):
        self.msg = msg
        self.order_id = order_id

    def __str__(self):
        return str(self.msg)


class AlreadyPaid(Exception):
    def __init__(self, order_id: str):
        self.order_id = order_id



import hashlib
from django_pay2.settings import payment_settings
from django_pay2.payment_methods import PaymentForm
from django_pay2.utils import remove_trivial_zeros


class FreeKassaApi:
    form_url = "https://www.free-kassa.ru/merchant/cash.php"

    def __init__(self, merchant_id, secret_word_1, secret_word_2):
        self.merchant_id = merchant_id
        self.secret_word_1 = secret_word_1
        self.secret_word_2 = secret_word_2

    def generate_payment_method(self, amount, order_id):
        return PaymentForm(
            action=self.form_url,
            fields={
                "m": self.merchant_id,
                "oa": str(amount),
                "o": str(order_id),
                "s": self.generate_form_sign(amount, order_id),
            },
        )

    def generate_form_sign(self, amount, order_id):
        hash_str = f"{self.merchant_id}:{amount}:{self.secret_word_1}:{order_id}"
        return hashlib.md5(hash_str.encode("utf-8")).hexdigest()

    def generate_notify_sign(self, amount, order_id):
        hash_str = f"{self.merchant_id}:{remove_trivial_zeros(amount)}:{self.secret_word_2}:{order_id}"
        return hashlib.md5(hash_str.encode("utf-8")).hexdigest()


def get_api():
    return FreeKassaApi(
        payment_settings.FREE_KASSA.merchant_id,
        payment_settings.FREE_KASSA.secret_word_1,
        payment_settings.FREE_KASSA.secret_word_2,
    )



from .serializers import FreeKassaNotifySerializer


def handle_notify(data):
    serializer = FreeKassaNotifySerializer(
        data={
            "merchant_id": data.get("MERCHANT_ID"),
            "amount": data.get("AMOUNT"),
            "order": data.get("MERCHANT_ORDER_ID"),
            "sign": data.get("SIGN"),
        }
    )
    serializer.is_valid(raise_exception=True)
    payment = serializer.validated_data["order"]
    payment.accept()
    return payment



from django.conf import settings

from .api import TinkoffApi


def get_tinkoff_api() -> TinkoffApi:
    return TinkoffApi(
        settings.TINKOFF_TERMINAL_KEY,
        settings.TINKOFF_PASSWORD,
    )



from django_pay2.decorators import handle_debug
from django_pay2.models import Payment

from .api import get_api


@handle_debug
def create_coinpayments_payment(request, amount, desc, receiver, buyer_email, currency):
    payment = Payment.objects.create(amount=amount, receiver=receiver)
    api = get_api()
    return api.generate_payment_method(
        request, amount, currency, payment.id, buyer_email
    )



import logging

from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.views import generic
from django.views.decorators.csrf import csrf_exempt
from rest_framework.exceptions import ValidationError

from .services import handle_notify

logger = logging.getLogger(__name__)


@method_decorator(csrf_exempt, name="dispatch")
class NotifyView(generic.View):
    def post(self, request, *args, **kwargs):
        try:
            payment = handle_notify(request.POST)
        except ValidationError as exc:
            msg = list(exc.detail.values())[0][0]
            logger.info(f'Error with processing free kassa payment: {exc.detail}')
            return HttpResponse(f"NO: {msg}")
        logger.info(f'Successfully paid free kassa payment {payment.id}')
        return HttpResponse("YES")



import hashlib
from django_pay2.settings import payment_settings
from django_pay2.payment_methods import PaymentForm
from django.urls import reverse_lazy


class PerfectMoneyApi:
    form_url = "https://perfectmoney.is/api/step1.asp"

    notify_url = reverse_lazy("django_pay2:perfect_money:notify")

    def __init__(
        self,
        payee_usd_account,
        payee_eur_account,
        payee_name,
        success_url,
        failed_url,
        alternate_passphrase,
    ):
        self.payee_usd_account = payee_usd_account
        self.payee_eur_account = payee_eur_account
        self.payee_name = payee_name
        self.success_url = success_url
        self.failed_url = failed_url
        self.alternate_passphrase = alternate_passphrase

    def generate_payment_method(self, request, amount, currency, order_id):
        return PaymentForm(
            action=self.form_url,
            fields={
                "PAYEE_ACCOUNT": self.get_payee(currency),
                "PAYEE_NAME": self.payee_name,
                "PAYMENT_AMOUNT": f"{amount:.2f}",
                "PAYMENT_UNITS": currency,
                "PAYMENT_ID": str(order_id),
                "STATUS_URL": request.build_absolute_uri(self.notify_url),
                "PAYMENT_URL": request.build_absolute_uri(self.success_url),
                "NOPAYMENT_URL": request.build_absolute_uri(self.failed_url),
                "BAGGAGE_FIELDS": "",
            },
        )

    def calculate_sign(
        self,
        payment_id,
        payment_amount,
        payment_units,
        payment_batch_num,
        payer_account,
        timestamp,
    ):
        hashed_pass = (
            hashlib.md5(self.alternate_passphrase.encode("utf-8")).hexdigest().upper()
        )
        hash_list = [
            payment_id,
            self.get_payee(payment_units),
            payment_amount,
            payment_units,
            payment_batch_num,
            payer_account,
            hashed_pass,
            timestamp,
        ]
        sign = hashlib.md5(":".join([str(item) for item in hash_list]).encode("utf-8"))
        return sign.hexdigest().upper()

    def get_payee(self, currency):
        return self.payee_usd_account if currency == "USD" else self.payee_eur_account


def get_api():
    return PerfectMoneyApi(
        payment_settings.PERFECT_MONEY.payee_usd_account,
        payment_settings.PERFECT_MONEY.payee_eur_account,
        payment_settings.PERFECT_MONEY.payee_name,
        payment_settings.PERFECT_MONEY.success_url,
        payment_settings.PERFECT_MONEY.failed_url,
        payment_settings.PERFECT_MONEY.alternate_passphrase,
    )



import hashlib
import hmac
from django_pay2.settings import payment_settings
from django_pay2.payment_methods import PaymentRedirect
from django.urls import reverse_lazy
import requests
from urllib.parse import urlencode
from django_pay2.exceptions import CreatePaymentError


class CoinPaymentsApi:
    url = "https://www.coinpayments.net/api.php"

    notify_url = reverse_lazy("django_pay2:coinpayments:notify")

    def __init__(self, public_key, private_key, ipn_secret, success_url, cancel_url):
        self.public_key = public_key
        self.private_key = private_key
        self.ipn_secret = ipn_secret
        self.success_url = success_url
        self.cancel_url = cancel_url

    def generate_payment_method(self, request, amount, currency, order_id, buyer_email):
        response = self.request(
            {
                "cmd": "create_transaction",
                "amount": str(amount),
                "currency1": currency,
                "currency2": currency,
                "buyer_email": buyer_email,
                "invoice": str(order_id),
                "ipn_url": request.build_absolute_uri(self.notify_url),
                "success_url": request.build_absolute_uri(self.success_url),
                "cancel_url": request.build_absolute_uri(self.cancel_url),
            }
        )
        return PaymentRedirect(response["checkout_url"])

    def calculate_ipn_hmac(self, encoded_body):
        return hmac.new(
            self.ipn_secret.encode("utf-8"),
            encoded_body.encode("utf-8"),
            hashlib.sha512,
        ).hexdigest()

    def request(self, data):
        data.setdefault("version", 1)
        data.setdefault("key", self.public_key)

        encoded_body = self._encode_body(data)
        headers = {
            "HMAC": self.create_hmac(encoded_body),
        }
        response = requests.post(self.url, data=data, headers=headers)
        response.raise_for_status()
        res_data = response.json()
        if res_data["error"] != "ok":
            raise CreatePaymentError(res_data["error"])
        return res_data["result"]

    def create_hmac(self, encoded_body):
        return hmac.new(
            bytes(self.private_key, "utf-8"),
            encoded_body.encode("utf-8"),
            hashlib.sha512,
        ).hexdigest()

    @staticmethod
    def _encode_body(body: dict) -> str:
        return urlencode(body)


def get_api():
    return CoinPaymentsApi(
        payment_settings.COINPAYMENTS.public_key,
        payment_settings.COINPAYMENTS.private_key,
        payment_settings.COINPAYMENTS.ipn_secret,
        payment_settings.COINPAYMENTS.success_url,
        payment_settings.COINPAYMENTS.cancel_url,
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_pay2',
 'django_pay2.contrib.rest_framework',
 'django_pay2.migrations',
 'django_pay2.providers',
 'django_pay2.providers.coinpayments',
 'django_pay2.providers.debug',
 'django_pay2.providers.free_kassa',
 'django_pay2.providers.payeer',
 'django_pay2.providers.perfect_money',
 'django_pay2.providers.qiwi',
 'django_pay2.providers.sberbank',
 'django_pay2.providers.tinkoff',
 'django_pay2.providers.tinkoff.api']

package_data = \
{'': ['*'],
 'django_pay2': ['static/css/*',
                 'static/img/*',
                 'static/js/*',
                 'templates/django_pay2/*']}

install_requires = \
['Django>=3.2.7,<4.0.0',
 'django-ipware>=4.0.0,<5.0.0',
 'djangorestframework>=3.12.4,<4.0.0',
 'requests>=2.26.0,<3.0.0']

setup_kwargs = {
    'name': 'django-pay2',
    'version': '1.4.0',
    'description': 'Интеграция платежных систем для Django',
    'long_description': "=============================\nDjangoPay2\n=============================\n\n.. image:: https://travis-ci.org/la1t/django_pay2.svg?branch=master\n    :target: https://travis-ci.org/la1t/django_pay2\n\nИнтеграция платежных систем для Django\n\nQuickstart\n----------\n\nУстановите DjangoPay2::\n\n    pip install django_pay2\n\nДобавьте приложение в `INSTALLED_APPS`:\n\n.. code-block:: python\n\n    INSTALLED_APPS = (\n        ...\n        'django_pay2',\n        ...\n    )\n\nДобавьте Django Pay's URLы в urlpatterns:\n\n.. code-block:: python\n\n\n    urlpatterns = [\n        ...\n        path('payments/', include('django_pay2.urls')),\n        ...\n    ]\n\nПлатежи\n-----------------\n\nЛюбой платеж создается методом create_<название-платежной-системы>_payment. Метод возвращает объект типа PaymentMethod. Существует два типа\nPaymentMethod — PaymentForm и RedirectMethod. Первый должен сериализовываться и отправляться на front. Второй можно либо также сериализовать\nи отправить на фронт, либо возвратить редирект.\n\nТак же метод создает объект типа Payment. Эта модель отвечает за сохранение информации о платеже, а так же об объекте-инициаторе платежа. \nПосле успешной оплаты он отправляет сигнал `payment_received` привязанному объекту. \n\nTODO\n--------\n\n- [*] Сериализаторы для PaymentForm и RedirectMethod\n- [ ] Интеграция с free_kassa\n- [ ] `handle_form_debug` проверяет, что ему пришла валидная форма\n- [*] Переписывание тестов на pytest\n- [ ] Инструкция по написанию собственных провайдеров\n- [ ] Инструкция по добавлению и настройке каждого провайдера\n- [ ] Более подробная инструкция по использованию\n- [*] Инструкция по увеличению версии\n\n\nРазработка\n----------\n\nУстановка development зависимостей\n\n::\n  pip install -r requirements_dev.txt\n\nЗапуск тестов\n\n::\n  tox\n\nВыпуск\n--------\n\n1. Установить новую версию в `pyproject.toml' и `__init__.py` файлах\n2. `poetry publish --build`\n",
    'author': 'Anatoly Gusev',
    'author_email': 'gusev.tolia@yandex.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/la1t/django_pay',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<3.10',
}


setup(**setup_kwargs)
